#!/usr/bin/perl

# Usage: grabRegionInventorySnapshot.pl -object <objectName> -operation <op> -entityType <entityType> -oldScript <input PowerShell script> -newScript <script you want to create>

use strict;
use Getopt::Std;
use Getopt::Long;

# Perl script to extract commands related to a specific entity from large PowerShell InventorySnapshot.ps1 file
# For example, if you have an InventorySnapshot.ps1 file and you only want to restore cluster foo,
# you can use this script as follows:
# grabRegionInventorySnapshot.pl -object "foo" -operation "ALL" -entityType cluster -oldScript InventorySnapshot.ps1 -newScript "foo.ps1"
# This will grab only commands related to restoring cluster foo and put them in "foo.ps1"


## Make sure arguments are correct
my $numArgs = $#ARGV + 1;

sub Usage {
    print "###";
    print "Perl script to extract commands related to a specific entity from large PowerShell InventorySnapshot.ps1 file\n";
    print "For example, if you have an InventorySnapshot.ps1 file and you only want to restore cluster foo,\n";
    print "you can use this script as follows:\n";
    print "grabRegionInventorySnapshot.pl -object \"foo\" -operation \"ALL\" -entityType cluster -oldScript InventorySnapshot.ps1 -newScript \"foo.ps1\"\n";
    print "This will grab only commands related to restoring cluster foo and put them in \"foo.ps1\"\n";
    print "###";
    die "Usage: grabRegionInventorySnapshot.pl -object <objectName> -operation <op> -entityType <entityType> -oldScript <input PowerShell script> -newScript <script you want to create>";
}

sub ValidateInputs {
    my ($objectName,$operation,$entityType,$oldScript,$newScript) = @_;
    if ($objectName == -1) {
	die "Please supply an object name (-object)\n";
    }
    if ($operation == -1) {
	die "Please supply an operation name (-operation)\n";
    }
    if ($entityType == -1) {
	die "Please supply an entity type (-entityType)\n";
    }
    if ($oldScript == -1) {
	die "Please supply an input PowerShell script that should be edited (-oldScript)\n";
    }
    if ($newScript == -1) {
	die "Please supply a name for the output PowerShell script (-newScript)\n";
    }
}

sub main {
    my $help = 0;
    my $infile;
    my $objectName = -1;
    my $operation = -1;
    my $entityType = -1;
    my $oldScript = -1;
    my $newScript = -1;

    GetOptions('h' => \$help,
	       'object=s' => \$objectName,
	       'operation=s' => \$operation,
	       'entityType=s' => \$entityType,
	       'oldScript=s' => \$oldScript,
	       'newScript=s' => \$newScript);

    if ($help == 1) {
	&Usage()
    }
    &ValidateInputs($objectName,$operation,$entityType,$oldScript,$newScript);
    
    if ($operation eq "ALL") {
	$operation = ".*";
    }

    my $objectStart = "START " . $operation . " " . $entityType . ": " . $objectName;
    my $objectEnd = "END " . $operation . " " . $entityType .  ": " . $objectName;

    print "objectStart is $objectStart\n";
    print "objectEnd is $objectEnd\n";

    open INFD, "<$oldScript" or  die("Could not open $oldScript for reading");
    open OUTFD, ">$newScript" or die ("Could not open $newScript for writing");


    my $inRegion = 0;
    my $currLine;
    my $matchCounter = 0;
    my $lineCounter = 0;
    while (<INFD>) {
	$currLine = $_;
	# chomp($currLine);
	if ($currLine =~ /$objectStart/) {
	    # Ok, we've detected the relevant operation. Start getting stuff.
	    # Increment $inRegion to indicate that we should start grabbing the output
	    # and writing it to a file. We use a counter here instead of a boolean
	    # in case we have nested operations to the same object. Each new region
	    # we enter should increment this counter
	    print $currLine;
	    print OUTFD $currLine;
	    $inRegion += 1;
	    $matchCounter += 1;
	}
	elsif ($currLine =~ /$objectEnd/ && ($inRegion > 0)) {
	    # Ok, we've detected the end of the relevant region. Stop getting stuff.
	    # Also, decrement the region counter. If we have nested regions, we will
	    # still be writing the lines toa file.
	    print $currLine;
	    print OUTFD $currLine;
	    $inRegion -= 1;
	    $matchCounter += 1;
	}
	elsif ($inRegion > 0) {
	    print $currLine;
	    print OUTFD $currLine;
	    $matchCounter += 1;
	}
    } # while <INFD>
    close INFD;
    close OUTFD;

    # convert outfile to DOS since you will be running it in PowerShell
    system("unix2dos","$newScript");
}

main;
